import {
  convertLineBreaksIntoSpaces,
  ignoreSpacesAfterLineBreak,
  ignoreSpacesFollowingSpace,
  trim,
} from './helpers';

describe('ignoreSpacesAfterLineBreak', () => {
  test('1', () => {
    expect(ignoreSpacesAfterLineBreak('\n ')).toEqual('\n');
  });
  test('2', () => {
    expect(ignoreSpacesAfterLineBreak('\n   ')).toEqual('\n');
  });
  test('3', () => {
    expect(ignoreSpacesAfterLineBreak(' \n   ')).toEqual(' \n');
  });
  test('4', () => {
    expect(ignoreSpacesAfterLineBreak('  \n   ')).toEqual('  \n');
  });
});

describe('convertLineBreaksIntoSpaces', () => {
  test('1', () => {
    expect(convertLineBreaksIntoSpaces('\n')).toEqual(' ');
  });
  test('2', () => {
    expect(convertLineBreaksIntoSpaces('  \n\n  ')).toEqual('      ');
  });
  test('3', () => {
    expect(convertLineBreaksIntoSpaces(' \n \n ')).toEqual('     ');
  });
});

describe('ignoreSpacesFollowingSpace', () => {
  test('1', () => {
    expect(ignoreSpacesFollowingSpace([' ', ' ', ' '])).toEqual([' ', '', '']);
  });
  test('2', () => {
    expect(ignoreSpacesFollowingSpace([' ', ' foo ', ' '])).toEqual([
      ' ',
      'foo ',
      '',
    ]);
  });
  test('3', () => {
    expect(
      ignoreSpacesFollowingSpace([' ', '  foo  ', ' bar   baz  ', ' ']),
    ).toEqual([' ', 'foo ', 'bar baz ', '']);
  });
  test('4', () => {
    expect(
      ignoreSpacesFollowingSpace([
        ' ',
        '',
        ' ',
        '',
        '  foo  ',
        '',
        ' ',
        '',
        '  bar   baz  ',
        ' ',
        '',
        ' ',
      ]),
    ).toEqual([' ', '', '', '', 'foo ', '', '', '', 'bar baz ', '', '', '']);
  });
  test('5', () => {
    expect(
      ignoreSpacesFollowingSpace(['', '', ' ', ' foo  ', ' ', ' bar ', ' ']),
    ).toEqual(['', '', ' ', 'foo ', '', 'bar ', '']);
  });
});

describe('trim', () => {
  test('1', () => {
    expect(trim([' ', 'foo', ' '])).toEqual(['', 'foo', '']);
  });
  test('2', () => {
    expect(trim([' ', '  foo  ', ' '])).toEqual(['', 'foo', '']);
  });
  test('3', () => {
    expect(trim([' ', '', '  foo  ', '', ' '])).toEqual([
      '',
      '',
      'foo',
      '',
      '',
    ]);
  });
  test('4', () => {
    expect(trim(['', 'foo', ''])).toEqual(['', 'foo', '']);
  });
});
